//
//  NSMutableAttributedString+CZKit.swift
//  letaoshijie
//
//  Created by chaozheng on 2019/9/20.
//

import Foundation
import UIKit

public extension CZKit where Base: NSMutableAttributedString {
    
    @discardableResult
    func addAttribute(_ name: NSAttributedString.Key, value: Any, range: NSRange) -> CZKit {
        base.addAttribute(name, value: value, range: range)
        return self
    }
    
    @discardableResult
    func addAttributes(_ attrs: [NSAttributedString.Key : Any] = [:], range: NSRange) -> CZKit {
        base.addAttributes(attrs, range: range)
        return self
    }
    
    @discardableResult
    func append(_ attributedString: NSAttributedString) -> CZKit {
        base.append(attributedString)
        return self
    }
    
    @discardableResult
    func appends(_ attributedStrings: [NSAttributedString]) -> CZKit {
        attributedStrings.forEach{ base.append($0) }
        return self
    }
    
    @discardableResult
    func removeAttribute(_ name: NSAttributedString.Key, range: NSRange) -> CZKit {
        base.removeAttribute(name, range: range)
        return self
    }
    
    @discardableResult
    func font(font: UIFont, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.font, value: font, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.font, value: font, range: range)
        return self
    }
    
    @discardableResult
    func systemFont(fontSize: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.font, value: UIFont.systemFont(ofSize: fontSize), range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.font, value: UIFont.systemFont(ofSize: fontSize), range: range)
        return self
    }
    
    @discardableResult
    func boldSystemFont(fontSize: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.font, value: UIFont.boldSystemFont(ofSize: fontSize), range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.font, value: UIFont.boldSystemFont(ofSize: fontSize), range: range)
        return self
    }
    
    @discardableResult
    func paragraphStyle(_ paragraphStyle: NSParagraphStyle, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.paragraphStyle, value: paragraphStyle, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.paragraphStyle, value: paragraphStyle, range: range)
        return self
    }
    
    @discardableResult
    func foregroundColor(_ foregroundColor: UIColor, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.foregroundColor, value: foregroundColor, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.foregroundColor, value: foregroundColor, range: range)
        return self
    }
    
    @discardableResult
    func backgroundColor(_ backgroundColor: UIColor, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.backgroundColor, value: backgroundColor, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.backgroundColor, value: backgroundColor, range: range)
        return self
    }
    
    @discardableResult
    func ligature(_ ligature: Int, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.ligature, value: ligature, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.ligature, value: ligature, range: range)
        return self
    }
    
    @discardableResult
    func kern(_ kern: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.kern, value: kern, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.kern, value: kern, range: range)
        return self
    }
    
    /// 设置删除线
    @discardableResult
    func strikethroughStyle(_ strikethroughStyle: NSUnderlineStyle, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.strikethroughStyle, value: strikethroughStyle.rawValue, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.strikethroughStyle, value: strikethroughStyle, range: range)
        return self
    }
    
    /// 设置下划线
    @discardableResult
    func underlineStyle(_ underlineStyle: NSUnderlineStyle, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.underlineStyle, value: underlineStyle.rawValue, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.underlineStyle, value: underlineStyle, range: range)
        return self
    }
    
    @discardableResult
    func strokeColor(_ strokeColor: UIColor, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.strokeColor, value: strokeColor, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.strokeColor, value: strokeColor, range: range)
        return self
    }
    
    @discardableResult
    func strokeWidth(_ strokeWidth: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.strokeWidth, value: strokeWidth, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.strokeWidth, value: strokeWidth, range: range)
        return self
    }
    
    @discardableResult
    func shadow(_ shadow: NSShadow, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.shadow, value: shadow, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.shadow, value: shadow, range: range)
        return self
    }
    
    @discardableResult
    func textEffect(_ textEffect: String, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.textEffect, value: textEffect, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.textEffect, value: textEffect, range: range)
        return self
    }
    
    @discardableResult
    func attachment(_ attachment: NSTextAttachment, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.attachment, value: attachment, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.attachment, value: attachment, range: range)
        return self
    }
    
    @discardableResult
    func link(_ link: URL, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.link, value: link, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.link, value: link, range: range)
        return self
    }
    
    @discardableResult
    func baselineOffset(_ baselineOffset: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.baselineOffset, value: baselineOffset, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.baselineOffset, value: baselineOffset, range: range)
        return self
    }
    
    /// 设置下划线颜色
    @discardableResult
    func underlineColor(_ underlineColor: UIColor, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.underlineColor, value: underlineColor, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.underlineColor, value: underlineColor, range: range)
        return self
    }
    
    /// 删除线颜色
    @discardableResult
    func strikethroughColor(_ strikethroughColor: UIColor, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.strikethroughColor, value: strikethroughColor, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.strikethroughColor, value: strikethroughColor, range: range)
        return self
    }
    
    /// 设置字体倾斜度，取值为float，正值右倾，负值左倾
    @discardableResult
    func obliqueness(_ obliqueness: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.obliqueness, value: obliqueness, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.obliqueness, value: obliqueness, range: range)
        return self
    }
    
    /// 设置字体的横向拉伸，取值为float，正值拉伸 ，负值压缩
    @discardableResult
    func expansion(_ expansion: CGFloat, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.expansion, value: expansion, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.expansion, value: expansion, range: range)
        return self
    }
    
    @discardableResult
    func writingDirection(_ writingDirection: [Int], range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.writingDirection, value: writingDirection, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.writingDirection, value: writingDirection, range: range)
        return self
    }
    
    @discardableResult
    func verticalGlyphForm(_ verticalGlyphForm: Int, range: NSRange? = nil) -> CZKit {
        guard let range = range else {
            base.addAttribute(.verticalGlyphForm, value: verticalGlyphForm, range: NSMakeRange(0, base.length))
            return self
        }
        base.addAttribute(.verticalGlyphForm, value: verticalGlyphForm, range: range)
        return self
    }
}
